function T = ConstrainedRFC(data,cdata,clabels,nclus,CoRFC)
% T = ConstrainedRFC(data,cdata,clabels,nclus,CoRFC)
%  
% perform Constrained RF-clustering 
% 
% Input:
% data:  [nxd]: n objects in a d-dimensional space to be clustered
% cdata, clabels:   constraints
% nclus: number of desired clusters
% CoRFC: the Constrained RFC scheme
%        1 -> CoRFC-Shi
%        2 -> CoRFC-Zhu
%        3 -> CoRFC-RatioRF
%
% Output:
% T: the clustering

% for datasets larger that fastthr, subsample more
FASTthr = 256;
FASTsampling = 128;

% Step 1: Train RF with constrained data
% default parametrization
param = RFC_defaultParam; 
param.crit = 'mcgini';
if size(cdata,1) >= FASTthr
    param.sampling = FASTsampling;
end
% training RF with constrained dataset
param.nclas = length(unique(clabels));
rfc = RFC_classRFtrain(cdata,clabels,param);


% Second step: Computing RF-distance
switch CoRFC
    case 1
        param.dist = 1; % SHI
    case 2
        param.dist = 2; %ZHU
    case 3
        param.dist = 5; %RatioRF
end
% getting distance and clustering
CI = RFC_getRFClusInfo(data,rfc,param);
DD = RFC_getRFDist(data,rfc,param,CI);

% Third step: Clustering
T = RFC_disClus(DD,nclus,3);

