function t = RFC_SSerttreetrain(Xdata,Xsulab,param,liv)
% t = RFC_SSerttreetrain(Xdata,Xsulab,param,liv)
%  
% train tree with semisupervised information
% In a node: if labels are present, use gini, otherwise use
% Extremely Randomized Trees:

% P. Geurts et al: "Extremely randomized trees",
% Mach. Learn., 63 (1), 3–42, 2006. 
%
% Random Forest Clustering library
% (c) Manuele Bicego 2021
%
% Xdata: [nxd]: n objects in a d-dimensional space

global idn

n = size(Xdata,1);
allf = param.allf;
if param.featsubset == 0
    featsubset = length(param.allf);
else
    featsubset = param.featsubset;
end
fss = randperm(length(allf));
fss = allf(fss(1:featsubset));
t.idn = idn;
t.liv = liv;


nsul = unique(Xsulab);
nsul(nsul == -1) = [];

ndxsul = (Xsulab~=-1);
tmpsulab = Xsulab(ndxsul);
suXdata = Xdata(ndxsul,:);

% criterion at node
stopsplit = 0;
bestt = nan;
if liv > param.maxDepth
    % max length
    stopsplit = 1;
elseif n == 1
    % leave with one element
    stopsplit = 1;
% elseif (size(unique(data(objs,allf),'rows'),1) == 1) 
%     % all elements equal
%     stopsplit = 1;
elseif length(find(std(Xdata)>1e-10)) == 0
    % all elements equal
    stopsplit = 1;
else
    %fss = randperm(length(allf));
    %fss = allf(fss(1:featsubset));
    % random feature
    %bestf = fss(randi(length(fss)));
    
    % if there are at least two labels of different classes then use GINI,
    % otherwise use ERT

    
    if length(nsul) <= 1 | (length(find(std(suXdata(:,fss))>1e-10))==0)
        % all unlabeled or only one class --> ERT
        buoni = find(std(Xdata)>1e-10);
        a = randi(length(buoni));
        bestf = buoni(a);
        % random pick of splitting value
        [xi,bestI] = sort(Xdata(:,bestf));
        th = xi(1) + (xi(end) - xi(1))*rand;
        a = find(xi>th);
        if isempty(a)
            % all equal -> just split in the middle
            bestj = round(length(xi)/2);
        else
            bestj = a(1)-1;
        end
        bestt = th;%mean(xi(bestj:bestj+1));
        XdataLL = Xdata(bestI(1:bestj),:);
        XdataRR = Xdata(bestI(bestj+1:end),:);
    else
        % split with GINI
        sn = length(tmpsulab);
        Rt = RFC_classtreevalue(tmpsulab,param.crit);
        
        bestdiff = -inf;
        bestf = [];
        bestt = [];
        bestj = [];
        bestI = [];
        for i=fss
            % sort the data along feature i:
            [xi,I] = sort(suXdata(:,i));
            % tmpx = newdata(objs(I),:);
            tmplab = tmpsulab(I);
            % run over all possible splits:
            % every split should involve at least two points
            for j=1:sn-1
                splitest = (xi(j) ~= xi(j+1));
                % skip equal points
                if splitest
                    RtL = RFC_classtreevalue(tmplab(1:j),param.crit);
                    nl = length([1:j]);
                    RtR = RFC_classtreevalue(tmplab(j+1:end),param.crit);
                    nr = length([j+1:n]);
                    %diff = Rt - nr*RtR - nl*RtL;
                    diff = Rt - nr*RtR/(nr+nl) - nl*RtL/(nr+nl);
                    if diff > bestdiff
                        bestdiff = diff;
                        bestf = i;
                        bestj = j;
                        bestt = mean(xi(j:j+1));
                        bestI = I;
                    end
                end
            end
        end
        Xsplitfea = Xdata(:,bestf)<bestt;
        XdataLL = Xdata(Xsplitfea,:);
        Xsplitfea2 = 1:n;
        Xsplitfea2(Xsplitfea) = [];
        XdataRR = Xdata(Xsplitfea2,:);
    end
end
%[idn bestt]

if stopsplit
    % leave
    t.l = [];
    t.r = [];
    t.sizet = n;
    t.nleaves = 1;
    t.maxdepth = liv;
    t.Mgain = 0;
    t.nInternals = 0;
    t.maxidn = idn;
else
    t.bestf = bestf;
    t.bestt = bestt;
    t.Mgain = 1;
    idn = idn+1;
    t.l = RFC_erttreetrain(XdataLL,param,liv+1);
    idn = idn+1;
    t.r = RFC_erttreetrain(XdataRR,param,liv+1);
    t.sizet = n;
    t.nleaves = t.l.nleaves + t.r.nleaves; % due foglie, una radice
    t.maxdepth = max(t.l.maxdepth,t.r.maxdepth);
    t.nInternals = t.l.nInternals + t.r.nInternals +1;
    t.maxidn = max( t.l.maxidn, t.r.maxidn);
end

