% Demo for Constrained Random Forest Clustering
% as described in the submitted paper
% "Extending Random Forest-Clustering schemes to work with 
% partition-level constraints"
% by Manuele Bicego and Hafiz Ahmad Hassan
% submitted to ICPR 2024
% 
% (c) Manuele Bicego 
% Last update: Apr 2024
% 

clear all
close all


% require RFC_lib, adding to path
addpath('./RFC_lib');


% loading data 
% Example: the iris dataset
rng('default'); 
load('iris.mat');
nclus = 3;


% generating partition level constraints
p = 0.2;  %percentage of constraints
[cdata, clabels] = sampleConstraints(data,labels,p);


% The proposed approach: uncomment the version you want to try
CoRFC = 1; % 1 -> CoRFC-Shi
% CoRFC = 2; % 2 -> CoRFC-Zhu
% CoRFC = 3; % 3 -> CoRFC-RatioRF
T = ConstrainedRFC(data,cdata,clabels,nclus,CoRFC);

% Checking the results
% in labels we have the true labeling
ARI = RandIndex(T,labels);

fprintf('Adjusted Rand Index: %.4f\n',ARI);


